// (c) Tim Fechtner. Licenced under GPL 2 (yes, version 2 only!).

#include "radiostation.h"

#include "settings_general.h"
#include "settings_stream.h"
#include <ksharedconfig.h>
#include <ksharedptr.h>
#include <kstandarddirs.h>
#include <KConfig>
#include <ktemporaryfile.h>
#include <kconfigdialog.h>
#include <klocalizedstring.h>

#define AND  && 
#define OR  || 
#define NOT  ! 
#define EQUAL  == 

radioStation::radioStation(const QPointer<QObject> parent, const QPointer<QWidget> mainWidget, const QString &configFileName) : ripping(parent)
{
  m_mainWidget = mainWidget;
  setupConfigSystem(configFileName);
}

radioStation::radioStation(const QPointer<QObject> parent, const QPointer<QWidget> mainWidget, const QString &configFileName, const qlonglong &index) : ripping(parent)
{
  setIndex(index);
  m_mainWidget = mainWidget;
  setupConfigSystem(configFileName);
}

radioStation::~radioStation()
{
  delete config_skeleton;  //  We have to delete the object config_skeleton pointns on befor deleting the object shared_config points on because the first makes use of the second. As the object shared_config points on is deleted when it's pointer (shared_config is a special pointer managing this) is deleted (and that's done - as the pointer is a member of this class - after the descturctor call) we can make shure by deleting config_skeleton explicitly here that the object config_skeleton points on is deleted _befor_ the object shared_config points on. I don't want to relay on the QObject-parentship-system because I don't know _when_ exactly the child object is deleted, and this way it is shure.
  if ( settingsDialog != 0 ) delete settingsDialog; // muss (if) sein? nicht redundant
}

inline void radioStation::setupConfigSystem(const QString& configFileName)
{
  //var declarations
  QFile file;

  //make shure that internal_configFileName contains the path of a file which is readable and writeable. If possible, using "configFile" option passed to the contructor. If not, create a new (empty) one.
  file.setFileName(configFileName);
  // TODO: catch error message if file name is empty!
  if ( file.open(QIODevice::ReadWrite) ) { // file (which was passed through "configFileName" option in the constructor) working fine. Use it.
    file.close();
    internal_configFileName = configFileName;
  }
  else {                                   // create a new file
    KTemporaryFile newConfigFile;
    newConfigFile.setPrefix(KStandardDirs::locateLocal("appdata","stream_")); // TODO warum nicht locate() ?
    newConfigFile.setSuffix("_rc");
    if ( !newConfigFile.open() ) {
        // handle error...
    }
    newConfigFile.setAutoRemove(false);
    internal_configFileName = newConfigFile.fileName();
    newConfigFile.close();
  };

  // setup configuration system
  shared_config = KSharedConfig::openConfig(internal_configFileName, KConfig::SimpleConfig);
  config_skeleton = new settings_stream(shared_config); // no constructor where I can pass a parent
  // config_skeleton->setParent(this); Here I don't use QObject's parent-child-system but delete the object
  // in the destructor. See there for further explications.
}

QString radioStation::configFileName() const
{
  return internal_configFileName;
}

QString radioStation::streamName() const
{
  return config_skeleton->info_streamName();
}

void radioStation::setStreamName(const QString& streamName)
{
  if (config_skeleton->info_streamName() != streamName)
  {
    config_skeleton->setInfo_streamName(streamName);
    config_skeleton->writeConfig();
    emit streamNameChanged(index(), streamName);
  };
}

QString radioStation::serverName() const
{
  return config_skeleton->info_serverName();
}

void radioStation::setServerName(const QString& serverName)
{
  if (config_skeleton->info_serverName() != serverName)
  {
    config_skeleton->setInfo_serverName(serverName);
    config_skeleton->writeConfig();
    emit streamNameChanged(index(), serverName);
  };
}

qint64 radioStation::bitrate() const
{
  return config_skeleton->info_bitrate();
}

void radioStation::setBitrate(qint64 bitrate)
{
  // config_skeleton makes shure that no value < -1 is stored
  if (config_skeleton->info_bitrate() != bitrate)
  {
    config_skeleton->setInfo_bitrate(bitrate);
    config_skeleton->writeConfig();
    emit bitrateChanged(index(), bitrate);
  };
}

qint64 radioStation::metaInterval() const
{
  return config_skeleton->info_metaInterval();
}

void radioStation::setMetaInterval(qint64 metaInterval)
{
  // config_skeleton makes shure that no value < -1 is stored
  if (config_skeleton->info_metaInterval() != metaInterval)
  {
    config_skeleton->setInfo_metaInterval(metaInterval);
    config_skeleton->writeConfig();
    emit metaIntervalChanged(index(), metaInterval);
  };
}

QString radioStation::serverUri() const
{
  return config_skeleton->serverUri();
}

QStringList radioStation::parameterList() const
{
  // variables
  QStringList parameters;

  //code
  //TODO: options who aren't in the kcfg's
  // calculate all parameters
  parameters += ripping::parameterList();
  parameters.append("-u");
  parameters.append(config_skeleton->advanced_userAgentString());  //TODO test if this works when the string is emempty
kDebug() << parameters << endl;
  parameters.append(QString("--xs_offset=%1")
                     .arg( config_skeleton->offset() )
                   );
kDebug() << "offset parameter:" << parameters.last() << endl;
  parameters.append(QString("--xs_search_window=%1:%2")
                     .arg( config_skeleton->searchWindow() / 2 )
                     .arg( (config_skeleton->searchWindow() / 2) + (config_skeleton->searchWindow() % 2) )
                   ); // this produces a search window which length corresponds exactly to the value in the settings. When the value is odd, the last half of the window is 1 ms longer.
  parameters.append(QString("--xs_silence_length=%1")
                     .arg( config_skeleton->silenceWindow() )
                   );
  if (config_skeleton->writeSingleFile())
    parameters.append("-a");    //TODO file name schemes for single files: append here

  if (config_skeleton->writeSplittedFiles())
  {
    //TODO file name schemes splitted files
    parameters.append(QString("--xs_padding=%1:%2")
                       .arg( config_skeleton->splittedFilesPostpad() )
                       .arg( config_skeleton->splittedFilesPrepad() )
                     );
  }
  else
    parameters.append("-A");

  parameters.append("-k");
  parameters.append(QString::number(config_skeleton->skipFirstXTracks()));

  if ( config_skeleton->advanced_useTimeout() )
  {
    parameters.append("-m");
    parameters.append(QString::number(config_skeleton->advanced_timeoutAfterXSeconds()));
  }

  if ( config_skeleton->truncateDuplicatesInIncomplete() )
    parameters.append("-T");

kDebug() << parameters << endl;
  return parameters;
}

void radioStation::displaySettingsDialog()
{
  // if the dialog object yet exists, we can display it directly and return.
  // (Should normally not happen, because we delete it after the use.)
  if ( KConfigDialog::showDialog(internal_configFileName) )  //using the file name of the config file
    return;                                                  // as name of the dialog -> this name is unique!

  // else we have to construct and configure it befor displaying it and return
  settingsDialog = new settings_stream_dialog(m_mainWidget, internal_configFileName, config_skeleton);
  connect( settingsDialog,
           SIGNAL( finished() ),
           this,
           SLOT( deleteSettingsDialogLater() )
         );
  settingsDialog->show();
}

void radioStation::deleteSettingsDialogLater()
{
  setError("");
// TODO showstopper: emit signals for changes for all the properties that the settings dialog can have changed!
// TODO showstopper: restart the ripping!
  if ( NOT settingsDialog.isNull() )  // Because the parent of the settings dialog is external,
    settingsDialog->deleteLater();    // somebody could have deleted it.
}
