// (c) Tim Fechtner. Licenced under GPL 2 (yes, version 2 only!).

#ifndef RADIOSTATION_H
#define RADIOSTATION_H

#include "ripping.h"
#include "settings_stream_dialog.h"



/** \brief This class rips radio streams following the data avaible through a config file.
*
* It uses via #settings_stream a config file (see property #configFileName) to determinate
* the URI of the stream and further options for the record (pregrap, postgrab, user agent string...).
*
* It provides the public slot <tt>void displaySettingsDialog()</tt>, who shows a dialog that is
* a graphical interface to the config file. */
class radioStation : public ripping
 {



     Q_OBJECT



     /** This property holds the path to the configuration file used by this object.
     *   You can give the path to the configuration file in the constructor (see there for
     *   further details). Once set there, it can't be changed anymore.
     *
     * \li \e read: <tt> QString configFileName() const </tt>
     *
     * \sa <tt> QString internal_configFileName </tt> */
     Q_PROPERTY(QString configFileName READ configFileName)



public:
     //contructor(s) and destructor(s)
     /** Constructor of the class.
     * @param parent     Sets the parent of this object.
     * @param mainWidget When an options dialog gets displayed, this widget is used as parent and the dialog
     *                   gets centerd to this Widget. 
     * @param configFileName Here you can give a (local!) path to a configuration file. See
     *                   class \e QFile::setName for
     *                   details about which format is valid. If the file doesn't exist, the object tries to create
     *                   it. If this isn't possible (for example if the path is invalid or you don't have the rights
     *                   or you gave the name of a directory or.. or if you didn't give any path),
     *                   then a new configuration file is created in the appdata directory set in KDE, with
     *                   the name stream_??????_rc with the six ? replaced by random letters. You can obtain the path
     *                   of the really used file by the property #configFileName. */
     radioStation(const QPointer<QObject> parent = 0, const QPointer<QWidget> mainWidget = 0, const QString &configFileName = QString());
     /** Conveniance constructor of the class.
     * @param parent     Sets the parent of this object.
     * @param mainWidget When an options dialog gets displayed, this widget is used as parent and the dialog
     *                   gets centerd to this Widget. 
     * @param configFileName Here you can give a (local!) path to a configuration file. See
     *                   class \e QFile::setName for
     *                   details about which format is valid. If the file doesn't exist, the object tries to create
     *                   it. If this isn't possible (for example if the path is invalid or you don't have the rights
     *                   or you gave the name of a directory or.. or if you didn't give any path),
     *                   then a new configuration file is created in the appdata directory set in KDE, with
     *                   the name stream_??????_rc with the six ? replaced by random letters. You can obtain the path
     *                   of the really used file by the property #configFileName.
     * @param index      Initializes the property #index. */
     radioStation(const QPointer<QObject> parent, const QPointer<QWidget> mainWidget, const QString &configFileName, const qlonglong &index);
     /** The destructor. */
     virtual ~radioStation();

     //properties
     /** See property #configFileName. */
     QString configFileName() const;
     /** Reimplemented virtual function that uses the file #configFileName (via settings_stream)
     *   instead of a class member. */
     virtual QString streamName() const;
     /** Reimplemented virtual function that uses the file #configFileName (via settings_stream)
     *   instead of a class member. */
     virtual QString serverName() const;
     /** Reimplemented virtual function that uses the file #configFileName (via settings_stream)
     *   instead of a class member. */
     virtual qint64 bitrate() const;
     /** Reimplemented virtual function that uses the file #configFileName (via settings_stream)
     *   instead of a class member. */
     virtual qint64 metaInterval() const;



public slots:
     void displaySettingsDialog();



protected:
     //properties
     /** Reimplemented virtual function that uses the file #configFileName (via settings_stream)
     *   instead of a class member. */
     virtual void setStreamName(const QString& streamName);
     /** Reimplemented virtual function that uses the file #configFileName (via settings_stream)
     *   instead of a class member. */
     virtual void setServerName(const QString& serverName);
     /** Reimplemented virtual function that uses the file #configFileName (via settings_stream)
     *   instead of a class member. */
     virtual void setBitrate(qint64 bitrate);
     /** Reimplemented virtual function that uses the file #configFileName (via settings_stream)
     *   instead of a class member. */
     virtual void setMetaInterval(qint64 metaInterval);

     //other stuff
     virtual QStringList parameterList() const;
     virtual QString serverUri() const;  // this pure virtual function from "ripping" must be implemented!

     QPointer<QWidget> m_mainWidget;
     QPointer<settings_stream> config_skeleton;
     KSharedConfig::Ptr shared_config;
     QPointer<settings_stream_dialog> settingsDialog;



private:
     void setupConfigSystem(const QString& configFile);
     /** Used internally to store the property #configFileName. */
     QString internal_configFileName;



private slots:
     void deleteSettingsDialogLater();
 }; 

#endif
