// (c) Tim Fechtner. Licenced under GPL 2 (yes, version 2 only!).

#ifndef RIPPING_H
#define RIPPING_H

#include "streamripper_base.h"

/** \brief This abstract class uses \e %streamripper to rip radio streams.
*
*   You can use the public slots <tt>virtual void startStreamripper()</tt> and <tt>void shutDown()</tt> to start
*   and stop \e %streamripper.
*
*   This class applies the options avaible through #settings_general when calling \e %streamripper
*   (see <tt>virtual QStringList parameterList() const</tt>).
*
*   The actual state of the %ripping process is avaible through a number of properties.
*
*   You have to implement <tt>virtual QString serverUri() const</tt> in inherited classes.
*
*   Furthermore this class provides the static functions <tt>static QString format_bitrate(long bitrate)</tt>
*   and <tt>static QString format_metaInterval(long metaInterval)</tt>. */
class ripping : public streamripper_base
 {



     Q_OBJECT



     /** This property holds the bitrate of the stream (in \e kbit/s) that is recorded by \e %streamripper. 
     *
     * Meaning:
     * \li <b> bitrate >= 1:</b> The bitrate has been recognized. The property holds the bitrate.
     * \li <b> (bitrate = 0) OR (bitrate = -1):</b> An error occured while recognizing the bitrate. (Maybe a \e %streamripper error or maybe a %KRadioRipper error while recognzing.)
     * \li <b> bitrate <= -2:</b> The property isn't defined. The bitrate has not (yet) been recognized.
     *
     * <b> Initialization value: </b> \e -2
     *
     * This property guards it's last value also when \e %streamripper terminates.
     *
     * \li \e read: \code virtual qint64 bitrate() const \endcode
     * \li \e notify: \code void bitrateChanged(qlonglong index, qint64 newBitrate) \endcode 
     *
     * \sa \code virtual void setBitrate(qint64 newBitrate) \endcode
     * \sa \code qint64 internal_bitrate \endcode
     * \sa \code static QString format_bitrate(long bitrate) \endcode */
     Q_PROPERTY(qint64 bitrate READ bitrate)



     /** This property holds the amount of data of the actual song (in \e byte),
     *   that has been recorded by \e %streamripper until now. 
     *
     * Meaning:
     * \li <b> dataSize >= 0:</b> The dataSize has been recognized. The property holds the dataSize.
     * \li <b> dataSize = -1:</b> An error occured while recognizing the dataSize. (Maybe a \e %streamripper error or maybe a %KRadioRipper error while recognzing.)
     * \li <b> dataSize <= -2:</b> The property isn't defined. The dataSize has not (yet) been recognized.
     *
     * <b> Initialization value: </b> \e -2 
     *
     * This property gets reseted when \e %streamripper terminates.
     *
     * \li \e read: \code virtual qint64 dataSize() const \endcode
     * \li \e notify: \code void dataSizeChanged(qlonglong index, qint64 newDataSize) \endcode 
     *
     * \sa \code virtual void setDataSize(qint64 dataSize) \endcode
     * \sa \code qint64 internal_dataSize \endcode */
     Q_PROPERTY(qint64 dataSize READ dataSize)



     /** This property holds the error message of the last error that has occured.
     *   It holds error messages from \e %streamripper or an error messages from KRadioRipper (e.g. that
     *   the \e %streamripper command wasn't found or the "save directory" isn't writable).
     *   The error message is human readable and localised.
     *
     * Meaning:
     * \li <b>an empty string:</b> No error since the last startStreamripper().
     * \li <b>else:</b> An error occured since the last startStreamripper and the property holds the error message.
     *
     * <b> Initialization value: </b> <EM> an empty string </EM>
     *
     * This property guards it's last value also when \e %streamripper terminates.
     *
     * This property gets reseted when you startStreamripper().
     *
     * \li \e read: \code virtual QString error() const \endcode
     * \li \e notify: \code void errorChanged(qlonglong index, QString newError) \endcode 
     *
     * \sa \code virtual void setError(const QString& newError) \endcode
     * \sa \code QString internal_error \endcode 
     * \sa \code QString helper_formatedErrorMessage(const QString& streamripperOutputLine) \endcode */
     Q_PROPERTY(QString error READ error)



     /** This property holds the interval in which metadata is send by the stream.
     *
     * Meaning:
     * \li <b> metaInterval >= 1:</b> The metaInterval has been recognized. The property holds the bitrate.
     * \li <b> (metaInterval = 0) OR (metaInterval = -1):</b> An error occured while recognizing the metaInterval. (Maybe a \e %streamripper error or maybe a %KRadioRipper error while recognzing.)
     * \li <b> metaInterval <= -2:</b> The property isn't defined. The metaInterval has not (yet) been recognized.
     *
     * <b> Initialization value: </b> \e -2
     *
     * This property guards it's last value also when \e %streamripper terminates.
     *
     * \li \e read: \code virtual qint64 metaInterval() const \endcode
     * \li \e notify: \code void metaIntervalChanged(qlonglong index, qint64 newMetaInterval) \endcode 
     *
     * \sa \code virtual void setMetaInterval(qint64 newMetaInterval) \endcode
     * \sa \code qint64 internal_metaInterval \endcode
     * \sa \code static QString format_metaInterval(long metaInterval) \endcode */
     Q_PROPERTY(qint64 metaInterval READ metaInterval)



     /** \e %streamripper can start a local relay server which provides the stream that is actually recorded
     *   with some seconds of delay to the local network. This relay server is also used by KRadioRipper to
     *   implement to hear to the streams. This property holds the port of the relay server.
     *
     * Meaning:
     * \li <b> relayPort >= 0:</b> There exists a relay server. The property holds the port of the relay server.
     * \li <b> relayPort = -1:</b> An error occured while recognizing the relay server port. (Maybe a \e %streamripper error or maybe a %KRadioRipper error while recognzing.)
     * \li <b> relayPort <= -2:</b> There doesn't exist a relay server.
     *
     * <b> Initialization value: </b> \e -2 
     *
     * This property gets reseted when \e %streamripper terminates.
     *
     * \li \e read: \code virtual qint64 relayPort() const \endcode
     * \li \e notify: \code void relayPortChanged(qlonglong index, qint64 newRelayPort) \endcode 
     *
     * \sa \code virtual void setRelayPort(qint64 relayPort) \endcode
     * \sa \code qint64 internal_relayPort \endcode */
     Q_PROPERTY(qint64 relayPort READ relayPort)



     /** This property holds the name of server from which the stream comes.
     *
     * Meaning:
     * \li <b>an empty string:</b> The name of the server hasn't (yet) been recognized.
     * \li <b>else:</b> The name of the server has been recognized and the property holds it.
     *
     * <b> Initialization value: </b> <EM> an empty string </EM>
     *
     * This property guards it's last value also when \e %streamripper terminates.
     *
     * \li \e read: \code virtual QString serverName() const \endcode
     * \li \e notify: \code void serverNameChanged(qlonglong index, QString newServerName) \endcode 
     *
     * \sa \code virtual void setServerName(const QString& newServerName) \endcode
     * \sa \code QString internal_serverName \endcode */
     Q_PROPERTY(QString serverName READ serverName)



     /** This property holds the name of the song that is actually recorded by \e %streamripper.
     *
     * Meaning:
     * \li <b>an empty string:</b> Actually \e %streamripper isn't recordeding.
     * \li <b>else:</b> Actually \e %streamripper is recordeding and the property holds the name of the song who's actually recorded.
     *
     * <b> Initialization value: </b> <EM> an empty string </EM>
     *
     * \li \e read: \code virtual QString song() const \endcode
     * \li \e notify: \code void songChanged(qlonglong index, QString newSong) \endcode 
     *
     * \sa \code virtual void setSong(const QString& song) \endcode
     * \sa \code QString internal_song \endcode */
     Q_PROPERTY(QString song READ song)



     /** This property holds the status of \e %streamripper (like "Buffering...", "Ripping...",
     *   "Shutting down..." ...) in a human readable and localised form.
     *
     * Meaning:
     * \li <b>an empty string:</b> Actually \e %streamripper isn't working.
     * \li <b>else:</b> Actually \e %streamripper is working and the property holds the actual status.
     *
     * <b> Initialization value: </b> <EM> an empty string </EM>
     *
     * \li \e read: \code virtual QString status() const \endcode
     * \li \e notify: \code void statusChanged(qlonglong index, QString newStatus); \endcode 
     *
     * \sa \code virtual void setStatus(const QString& newStatus) \endcode
     * \sa \code QString internal_status \endcode */
     Q_PROPERTY(QString status READ status)



     /** This property holds the name of the stream that is recorded by \e %streamripper.
     *
     * Meaning:
     * \li <b>an empty string:</b> The name of the stream hasn't (yet) been recognized.
     * \li <b>else:</b> The name of the stream has been recognized and the property holds it.
     *
     * <b> Initialization value: </b> <EM> an empty string </EM>
     *
     * This property guards it's last value also when \e %streamripper terminates.
     *
     * \li \e read: \code virtual QString streamName() const \endcode
     * \li \e notify: \code void streamNameChanged(qlonglong index, QString newStreamName) \endcode 
     *
     * \sa \code virtual void setStreamName(const QString& newStreamName) \endcode
     * \sa \code QString internal_streamName \endcode */
     Q_PROPERTY(QString streamName READ streamName)



 public:
     // contructor(s) and destructor(s)
     /** Constructor of the class.
     * @param parent Sets the parent of this object. */
     ripping(const QPointer<QObject> parent);
     /** The destructor. */
     virtual ~ripping();



     // static
     /** \li If the parameter "bitrate" is >= 1, this function returns a nicly
     *   format QString version of bitrate in the form "23 kbit/s".
     *   \li If the parameter is  0 or -1, then a message is returned that there
     *   was an error while recognizing the bitrate.
     *   \li If the parameter is < -1, then an empty string is returned.
     *
     *   All returned strings are fully localised: They are translated and use the
     *   locally used number format.
     *   @param bitrate: the bitrate (in kbit/s) that will be formated.
     *   \sa #bitrate */
     static QString format_bitrate(long bitrate);



     /** \li If the parameter "metaInterval" is >= 1, this function returns a nicly
     *   format QString version of metaInterval.
     *   \li If the parameter is  0 or -1, then a message is returned that there
     *   was an error while recognizing the meta data interval.
     *   \li If the parameter is < -1, then an empty string is returned.
     *
     *   All returned strings are fully localised: They are translated and use the
     *   locally used number format.
     *   @param metaInterval: the meta interval that will be formated.
     *   \sa #metaInterval */
     static QString format_metaInterval(long metaInterval);



     // properties
     /** See property #bitrate. */
     virtual qint64    bitrate()          const;
     /** See property #dataSize. */
     virtual qint64    dataSize()         const;
     /** See property #error. */
     virtual QString   error()            const;
     /** See property #metaInterval. */
     virtual qint64    metaInterval()     const;
     /** See property #relayPort. */
     virtual qint64    relayPort()        const;
     /** See property #serverName. */
     virtual QString   serverName()       const;
     /** See property #song. */
     virtual QString   song()             const;
     /** See property #status. */
     virtual QString   status()           const;
     /** See property #streamName. */
     virtual QString   streamName()       const;



 signals:
     // others
     /** This signal is emited when the \e %streamripper process enters in the state
     * <tt> QProcess::NotRunning</tt>.
     *
     * This differs from <tt>QProcess:finished()</tt>, which is only emited when the
     * process was yet running.
     *
     * But <tt>not_running</tt> is also emited after you've tried to start \e %streamripper
     * (#m_process has yet entered in state <tt>QProcess::Starting</tt>) but the binary could than not be
     * invoced (because is doesn't exit or the user hasn't execution rights or whatever). */
     void not_running();

     // properties
     /** See property #bitrate. */
     void bitrateChanged(qlonglong index, qint64 newBitrate);
     /** See property #dataSize. */
     void dataSizeChanged(qlonglong index, qint64 newDataSize);
     /** See property #error. */
     void errorChanged(qlonglong index, QString newError);
     /** See property #metaInterval. */
     void metaIntervalChanged(qlonglong index, qint64 newMetaInterval);
     /** See property #relayPort. */
     void relayPortChanged(qlonglong index, qint64 newRelayPort);
     /** See property #serverName. */
     void serverNameChanged(qlonglong index, QString newServerName);
     /** See property #song. */
     void songChanged(qlonglong index, QString newSong);
     /** See property #status. */
     void statusChanged(qlonglong index, QString newStatus);
     /** See property #streamName. */
     void streamNameChanged(qlonglong index, QString newStreamName);



 public slots:
     /** This slot sends the \e terminate signal to the \e %streamripper instance running in #m_process.
     *   \sa <tt>void not_running()</tt> */
     void shutDown();
     /** This function starts \e %streamripper (using streamripper_base::startStreamripper()) if
     *   it isn't yet running. 
     *   It uses settings_general::saveDirectory() as working directory.
     *   It resets the property #error and actualizes the property #status.
     *   \sa <tt>void not_running()</tt>*/
     virtual void startStreamripper();



 protected:
     /** This function reads the output of the \e %streamripper process and interpretates it.
     *   It keeps the property of this class up to date - so when you want to use this information,
     *   use the properties.
     *
     *   Internally for performance reasons it uses from the many lines it can recive
     *   only the \e last avaible information
     *   of each type. (E.g. when it recives 15 lines <em>"[Ripping] song title [data size]"</em> it uses only the
     *   last one to set #status, #song and #dataSize properties - but controlls all the lines befor if they
     *   contain data of other type like #bitrate information or so on, and if so, it uses it.)
     *   \sa <tt> QString helper_formatedErrorMessage(const QString& streamripperOutputLine) </tt> */
     virtual void interpretate_console_output(QStringList& stringList);
     /** This function provides the nessesary parameters for calling \e %streamripper (to really rip
     *   a radio stream and not only to get the version number of \e %streamripper) as avaible
     *   from serverUri() and settings_general - and some extra parameter (like to don't override temporary files). */
     virtual QStringList parameterList() const;
     /** This function provides the URI which should be ripped and is used by parameterList(). Implement it
     *   in inherited classes! */
     virtual QString serverUri() const = 0;
     /** Used internally to write the property #bitrate. */
     virtual void setBitrate(qint64 newBitrate);
     /** Used internally to write the property #dataSize. */
     virtual void setDataSize(qint64 dataSize);
     /** Used internally to write the property #error. */
     virtual void setError(const QString& newError);
     /** Used internally to write the property #metaInterval. */
     virtual void setMetaInterval(qint64 newMetaInterval);
     /** Used internally to write the property #relayPort. */
     virtual void setRelayPort(qint64 relayPort);
     /** Used internally to write the property #serverName. */
     virtual void setServerName(const QString& newServerName);
     /** Used internally to write the property #song. */
     virtual void setSong(const QString& song);
     /** Used internally to write the property #status. */
     virtual void setStatus(const QString& newStatus);
     /** Used internally to write the property #streamName. */
     virtual void setStreamName(const QString& newStreamName);



 private:
     // help methods (all inline)
     /** This function is used internally to make a human readable and nicly formated string
     *   of the error message that was displayed by \e %streamripper.
     *
     *   Actually this code is not yet written and the function just returns the unformated error message!
     *   @param streamripperOutputLine the output line (as written out by \e %streamripper ) that will be formated. */
     QString helper_formatedErrorMessage(const QString& streamripperOutputLine);
     /** This internal helper function used in interpretate_console_output() interpretates a specific part of \e %streamrippers output and sets the properties
     *   #song and #dataSize accordingly.
     *   @param my_line A string of the form <em>song title [    1kb]</em>. The units <em>B</em>, <em>KB</em>, <em>K</em>, <em>MB</em> and <em>M</em> are recognized for the data size (not case sensitive). */
     void helper_interpretate_metainfo_and_datasize(QString my_line);

     //properties
     /** Used internally to store the property #bitrate. */
     qint64 internal_bitrate;
     /** Used internally to store the property #dataSize. */
     qint64 internal_dataSize;
     /** Used internally to store the property #error. */
     QString internal_error;
     /** Used internally to store the property #metaInterval. */
     qint64 internal_metaInterval;
     /** Used internally to store the property #relayPort. */
     qint64 internal_relayPort;
     /** Used internally to store the property #serverName. */
     QString internal_serverName;
     /** Used internally to store the property #song. */
     QString internal_song;
     /** Used internally to store the property #status. */
     QString internal_status;
     /** Used internally to store the property #streamName. */
     QString internal_streamName;



 private slots:
     /** This slot is to catch the signal <tt> QProcess::error(QProcess::ProcessError error) </tt> from #m_process.
     *
     *   If the error is...
     *   \li <b><tt> QProcess::FailedToStart</tt>:</b> Make a test if either the invoked program is missing or you may have insufficient permissions to invoke the program. An appropriate error messessage is set in the property #error.
     *   \li <b><tt> QProcess::Crashed</tt>:</b> An appropriate error messessage is set in the property #error.
     *   \li All other errors can safly be ignored because they don't mean that \e %streamripper is terminated.
     *
     *   @param error The <tt> QProcess::ProcessError </tt> that has occured. */
     void errorOccured(QProcess::ProcessError error);
     /** This slot is to catch the signal <tt> QProcess::stateChanged(QProcess::ProcessState) </tt> from #m_process.
     *
     *   \li When the state has changed to <tt> QProcess::NotRunning </tt> than the properties get adjusted.
     *   \li When the state has changed to <tt> QProcess::Starting </tt> nothing happens because the properties get adjusted by the calling function.
     *   \li When the state has changed to <tt> QProcess::Running </tt> nothing happens because the properties will get adjusted by interpretate_console_output().
     *
     *   @param newState The new <tt> QProcess::ProcessState </tt> that has occured. */
     void streamripperStateChange(QProcess::ProcessState newState);
 }; 

#endif
