// (c) Tim Fechtner. Licenced under GPL 2 (yes, version 2 only!).

#ifndef STREAMRIPPER_BASE_H
#define STREAMRIPPER_BASE_H

#include "console_reader.h"


/** \brief This abstract class implements access to \e %streamripper.
*
*   To do so, it inherits from console_reader
*   and uses settings_general to find the \e %streamripper executable - see startStreamripper().
*
*   Furthermore it implements the property #index and the functions kill() and waitForFinished().
*/
class streamripper_base : public console_reader
 {



     Q_OBJECT



     /** This property has no function in this class. You can use it
     * for whatever purpose you want.
     *
     * This property is quite usefull when you
     * have many objects of a child class of this class and you want to identify the sender
     * of a signal - just emit \e index as parameter of the signal.
     * \li \e read: \code virtual qlonglong index() const \endcode
     * \li \e write: \code virtual void setIndex(const qlonglong& value) \endcode
     * \li \e reset: \code virtual void resetIndex() \endcode
     *               Sets \e index to \e 0. This function is also used to initialize the property
     *               in the constructor. So when you want a different default value for the initialization,
     *               just reimplement this function.
     * \li \e notify: \code indexChanged(qlonglong newIndex) \endcode 
     *
     * \sa \code qlonglong internal_index \endcode */
     Q_PROPERTY(qlonglong index READ index WRITE setIndex NOTIFY indexChanged)



 public:
     //contructor(s) and destructor(s)
     /** Constructor of the class.
     * @param parent Sets the parent of this object. */
     streamripper_base(const QPointer<QObject> parent = 0);
     /** Conveniance constructor of the class.
     * @param parent Sets the parent of this object.
     * @param index      Initializes the property #index. */
     streamripper_base(const QPointer<QObject> parent, const qlonglong &index);
     /** The destructor. */
     virtual ~streamripper_base();

     //properties
     /** See property #index. */
     virtual qlonglong index() const;
     /** See property #index. */
     virtual void      resetIndex();
     /** See property #index. */
     virtual void      setIndex(const qlonglong& value);



 signals:
     /** See property #index. */
     void indexChanged(qlonglong newIndex);




 public slots:
     /** Calls <tt>#m_process</tt><tt>.%kill()</tt>. */
     void kill();
     /** This slot starts a \e %streamripper instance using #m_process. It doesn't care about if
     *   #m_process is yet running or not.
     *   
     *   Uses:
     *   \li  <tt> settings_general::streamripperCommand() </tt> as command
     *   \li  <tt> QStringList parameterList() const </tt> as list of parameters */
     virtual void startStreamripper();
     /** Calls <tt>#m_process</tt><tt>.%waitForFinished()</tt>. This takes usually only some milliseconds. */
     void waitForFinished();



 protected:
     /** This abstract function is called by startStreamripper() to determinate the parameters
     *   with which \e %streamripper will be called.
     *   \returns A \c QStringList with the parameters. */
     virtual QStringList parameterList() const = 0;



 private:
     /** Used internally to store the property #index.*/
     qlonglong internal_index;
 }; 

#endif
