// (c) Tim Fechtner. Licenced under GPL 2 (yes, version 2 only!).

#ifndef STATIONLISTMODEL_H
#define STATIONLISTMODEL_H
 
#include <QAbstractTableModel>
#include <QPointer>
#include <QItemSelection>
#include <QList>
#include <Phonon/MediaSource>
#include <Phonon/MediaObject>
#include <Phonon/AudioOutput>
#include "radiostation.h"

/** This class handels radio stations. Internally, it uses the class radiostation
  * to handle the stations. It reads the list of configuration
  * files of radio stations from settings_general and represents these radio
  * stations. This class can also create new radio stations or delete existing
  * ones - and takes care of creating/changing/removing the corresponding
  * config files.
  *
  * This class inherits from \e QAbstractTableModel (and this way indirectly from
  * \e QAbstractItemModel). So it can be used as model for QT's model-view-programming.
  * Use a \e QTableView object to watch the data of this model.
  *
  * Attention: Use only one instance of this class at the same time! (Otherwise, the
  * same stream could be recorded by more than one instance at the same time - and 
  * written by all instances to the same file! That's bad! Also this class uses the config
  * files - and it is bad when more than one instance uses them...
  */
class stationlistModel : public QAbstractTableModel
 {
     Q_OBJECT
     /** This property holds the index (= row) of the stream who is played back.
     *   When the corresponding stream isn't recorded or relay server are disabled,
     *   of course there is also no playback.
     *
     *   Use playing() to get the index. When playback is disabled, you get -1, else
     *   the index (=row) of the stream. Attention: This doesn't mean that the stream
     *   is really played back - when he isn't recorded at the moment or relay server
     *   are disabled, there is no playback.
     *
     *   Use setPlaying(const int & index) to set the index. When you provide a valid
     *   index (= an existing row), then the property is set and the playback starts
     *   if possible. If playback isn't possible, the index is saved anyway, and for
     *   example when you start recording the stream later, then playback will start
     *   also. When you give an invalid index as argument, than playback is stopped
     *   and the property is set to -1. */
     Q_PROPERTY(int playing READ playing WRITE setPlaying)
     /** This property offers access to the UI-width of the columns that
     *   are provided by this model.
     *
     *   Technicaly it provides access to the corresponding information in 
     *   the central config file (using settings_general).
     *
     *   A widget which provides a view on this model should take care of
     *   this property and read and apply it in the constructor. In the destructor,
     *   it should write to this property (and this way to the config file)
     *   the actual values. This way it is guaranteed that the last state of the 
     *   UI is saved.
     *
     *   The property is a \e QList of \e quint64 values. For each column, it
     *   contains the width. However, in the config file the values aren't saved
     *   as a simple list of values, but each column has an individual entry
     *   for the width. This way it is guaranteed that, also when the model changes
     *   the order of columns, each column guards its width. Furthermore, no changes
     *   at the "view" widget are nessesary as this system works without that the
     *   view widget knows about the order of the columns and of there content.
     *
     *
     * \li \e read: QList\<quint64\> columnWidthList() const \endcode
     * \li \e write: void setColumnWidthList(QList\<quint64\> value) \endcode
     *
     * \sa #columnVisibilityList */
//     Q_PROPERTY(QList<quint64> columnWidthList READ columnWidthList WRITE setColumnWidthList DESIGNABLE false STORED false)  TODO
     /** This property is very similar to #columnWidthList (see there
     *   for details). However is guards not
     *   the \e width of the columns but \e if they are visible or not.
     *
     *   This value is independent from the column width, because also a not visible
     *   column has a width (which is bigger than 0!) - and with this width it will
     *   appear when it is made visible.
     *
     * \li \e read: QList\<bool\> columnVisibilityList() const \endcode
     * \li \e write: void setColumnVisibilityList(QList\<quint64\> value) \endcode
     *
     * \sa #columnWidthList */
//     Q_PROPERTY(QList<quint64> columnVisibilityList READ columnVisibilityList WRITE setColumnVisibilityList DESIGNABLE false STORED false)  TODO

 public:
     // contructor(s) and destructor(s)
     /** The constructor. */
     stationlistModel(const QPointer<QObject> parent = 0, const QPointer<QWidget> mainWidget = 0);
     virtual ~stationlistModel();

     enum columnInfoType {
                           columnHeaderTitle,
                           columnHeaderToolTip,
                           columnHeaderWhatsThis,
                           columnWidth,
                           setColumnWidth,
                           columnVisibility,
                           setColumnVisibility,
                           columnData,
                           columnDataToolTip,
                           columnDataWhatsThis
                         };
     /** Warnung: If accessing \e data, than the given \e line MUST be valid. If not, the function crashes!
     *    / "value" is the line where to read for columnData[...] types OR the value to set for write_[...]
     *    types.  
     *    Attention: this function is "const" (qualifier), but in reallity it can WRITE values! */
     QVariant columnInfo(const int & column, const columnInfoType & type, const qint64 & value = (-1)) const;

     // reimplementation of virtual function from QAbstractTableModel 
     /** Returns the number of rows.
     * @param parent As this is a table model (non-hirarchical!),
     * this parameter is ignored. */
     virtual int rowCount( const QModelIndex & parent = QModelIndex() ) const;
     /** Returns the number of columns.
     * @param parent As this is a table model (non-hirarchical!),
     * this parameter is ignored. */
     virtual int columnCount( const QModelIndex & parent = QModelIndex() ) const;
     /** Returns the data of a the spezified item.
     * @param index As this is a table model (non-hirarchical!),
     * only rows and columns of the index are relevant.
     * The parent is ignored. If the index is invalid, an invalid QVariant is returned.
     * @param role  The type of information that is requested. Supported are
     * Qt::DisplayRole (the data itself) and
     * Qt::ToolTipRole and Qt::WhatsThisRole. For other values,
     * an invalid \e QVariant is returned. */
     virtual QVariant data ( const QModelIndex & index, int role = Qt::DisplayRole ) const;
     virtual QVariant headerData(int section, Qt::Orientation orientation, int role) const;

     //properties
     int playing() const;

 signals:

 public slots:
     /** This slot adds a new radio station. */
     void addNewStation();
     void deleteStation(const int& index);
     /** This slot starts the recording of a stream.
     *   @param index You must pass the index (= the row) of the stream which you want to record.
     *                When you pass an invalid index, nothing happens. */
     void record(const int& index);
     /** This slot stops the recording of a stream.
     *   @param index You must pass the index (= the row) of the stream which you want to stop the recording.
     *                When you pass an invalid index, nothing happens. */
     void stopRecording(const int& index);
     /** If exactly one stream is selected, then this slot displays the config dialog.
     *   Else nothing happens. */
     void displayConfigDialog(const int& index);
     void setPlaying(int index);
     /** This conveniance function sets the property playing, but takes a QModelIndex as argument instead of an int.*/
     void setPlayingModelIndex(const QModelIndex & index);

 private:
     //properties
     int internal_playing;

     //other stuff
     /** This member helds the value of the port that is actually in
     *   reallity used for the playback. It should have the value -1 when there
     *   is no playback. Otherwise it should have a value >= 0 which is the port. */
     qint64 internal_actuallyPlaybackedPort;
     /** This is a QList of objects of the radiostation class
     *   used internally to handle the stations. */
     QList< QPointer<radioStation> > m_stationlist;  /* we need a pointer as list member
                                                      instead of radioStation itself
                                                      because we need the construct
                                                      radioStation with the constructor
                                                      (no other way to set the config file),
                                                      and when radioStation's
                                                      constructor is called, the config file
                                                      is opened and so on. I
                                                      don't think it's save to copy this
                                                      class - what's nessesary
                                                      when not using a pointer. */
     /** a pointer to the Phonon MediaObject used for playback. */
     QPointer<Phonon::MediaObject> m_mediaObject;
     QPointer<Phonon::AudioOutput> m_audioOutput;
     Phonon::Path m_path;
     QPointer<QWidget> m_mainWidget;

     // help methods
     void writeStationListToGeneralSettings();
     void connectSignalsAndSlots(QPointer<radioStation> m_stream);
     void actualizeAudioPlayer();

 private slots:
     void reloadStreamName(const qlonglong& stationIndex);
     void reloadStatus(const qlonglong& stationIndex);
     void reloadSong(const qlonglong& stationIndex);
     void reloadDataSize(const qlonglong& stationIndex);
     void reloadBitrate(const qlonglong& stationIndex);
     void reloadMetaInterval(const qlonglong& stationIndex);
     void reloadServerName(const qlonglong& stationIndex);
     void reloadRelayPort(const qlonglong& stationIndex);

     void process_new_mediaObjectState ( Phonon::State newstate);

 }; 

#endif
